//////////////////////////////////////////////////////////////
// Version 1.00 first release
//
// Version 1.01 Added event for humans to detect better humans
//              Added correspondencegame detection to detect better humans
//              Delete the elo rating from a playername IE: {2500} 

#include <cstdio>
#include <cstdlib>
#include <iostream>

#define MAXBUF_SIZE 4096
#define MAX_LIST 5000
#define MAX_NAME_LENGTH 100
#define MAX_TAG_LENGTH 512
#define MAX_TAGS_NUMBER 20

using namespace std;

struct nameRecordType
{
	char name[MAX_NAME_LENGTH];
	bool end_star;   //playername has to end with this string
	bool start_star; //playername has to start with this string
};

struct nameListType
{
	nameRecordType data[MAX_LIST];
	int aantal;
};

enum ePlayerType { UNKNOWN_PLAYER, COMP_PLAYER, HUMAN_PLAYER, COMP_MAYBE, COMP_BYE, ADVANCED_PLAYER };

enum eGameType { REST_VS_REST, COMP_VS_COMP, COMP_VS_HUMAN, COMP_VS_UNKNOWN, ADVANCED_GAME };

enum eEventType { REST_EVENT, ICC_EVENT, FICS_EVENT };

struct playerDataType {
	char name[MAX_TAG_LENGTH];
	char uppered[MAX_TAG_LENGTH];
	char stripped_and_uppered[MAX_TAG_LENGTH];
	char cleaned_from_rating[MAX_TAG_LENGTH];
   ePlayerType type;
};

struct tagListType {
	char data[MAX_TAGS_NUMBER][MAX_TAG_LENGTH];
	int aantal;
};

struct pgnType
{
	char event[MAX_TAG_LENGTH];
	playerDataType white;
	playerDataType black;
	char site[MAX_TAG_LENGTH];
	tagListType tags;
	char date_icc_fixed[MAX_TAG_LENGTH];
	eGameType eGameType;
	eEventType eventType;
	int year, month, day;
} pgngame;

char line[MAXBUF_SIZE];

nameListType computers;
nameListType computers_icc;
nameListType computers_fics;
nameListType computers_when_opponent_is;
nameListType comp_events;
nameListType humans;
nameListType humans_icc_titled;
nameListType humans_icc_rest;
nameListType humans_fics;
nameListType human_events;
nameListType advanced_chess;
nameListType adv_events;


FILE *file_comp_vs_comp;
FILE *file_comp_vs_human;
FILE *file_comp_vs_rest;
FILE *file_rest_vs_rest;
FILE *file_advanced_chess;

FILE *active;

//eats spaces
void ltrim(char *s)
{
   while (s[0] == '\n' ||
          s[0] == '\t' ||
          s[0] == ' ') 
   {
      strcpy(s, s+1);
   }
}

/////////////////////////// ////////////////////////
// rtrim
// 
// take a way \n, \t and ' ' on the end
// I had to trail also 10 and 13 because of ssdf.pgn
/////////////////////////// ////////////////////////
void rtrim(char* s)
{
   for (int lengte = (int)strlen(s)-1; lengte >= 0; lengte--)
   {
      if (s[lengte] == '\n' || 
		  s[lengte] == '\t' || 
		  s[lengte] == 10 ||
		  s[lengte] == 13 ||
		  s[lengte] == ' ')
      {
         s[lengte] = '\0';
      }
      else
      {
         break;
      }
   }
}

//trail away a certain character from a string
void rtrim(char* s, char c)
{
   for (int lengte = (int)strlen(s)-1; lengte >= 0; lengte--)
   {
      if (s[lengte] == c)
      {
         s[lengte] = '\0';
      }
      else
      {
         break;
      }
   }
}

//does the string <str> start with <cmp> ?
bool beginsWith(char* str, char* cmp)
{
   if (str == NULL) return false;

   //compare first part of 'str' with 'cmp'
   if (strncmp(str, cmp, strlen(cmp)) == 0)
   {
	  return true;
   }
   else
   {
	  return false;
   }
}

bool beginsWithAndDelete(char* &str, char* cmp)
{
	if (str == NULL) return false;

   //get lengte of string
   int comp_len = (int)strlen(cmp);

   //compare first part of 'str' with 'cmp'
   if (strncmp(str, cmp, comp_len) == 0)
   {
      //strings are equal
      //set the pointer at the start of the "value"
      //so we skip the compare string
      str += comp_len;

	  return true;
   }
   else
   {
	  return false;
   }
}

//deletes a the string <search< from the string <org>
void deleteAllFromString(char *org, char *search)
{
	int lengte = (int)strlen(search);
	char *pos;

	for (;;)
	{
		pos = strstr(org, search);
		if (pos == NULL) break;
		strcpy(pos, pos+lengte);
	} 
}

//does the string <str> ends with <cmp> ?
bool endsWith(char* str, char* cmp)
{
   //we have to look something at the end
   char* start = str + strlen(str) - strlen(cmp);

   return strcmp(start, cmp) == 0; 
}

//does the string <str> ends with <cmp> ?
//and delete this string totally when it does
bool endsWithAndDelete(char *str, char* cmp)
{
   //we have to look something at the end
   char* start = str + strlen(str) - strlen(cmp);

   if (strcmp(start, cmp) == 0) 
   {
	   *start = '\0';
	   return true;
   }
   else
   {
	   return false;
   }
}

//uppercase the string <str>
void upper(char *str)
{
	for (int i=0; i<(int)strlen(str); i++)
	{
		//convert to capitals
		if (str[i] >= 'a' && str[i] <= 'z') 
		{
			str[i] = str[i] - 'a' + 'A';
		}
	}
}

//strips '.', '/', ' ', '"',  ']' away from the playername and 
//converts everything to capitals
void stripCharsFromPlayerName(char* str)
{
	int newplace = 0;

	for (int i=0; i<(int)strlen(str); i++)
	{
		if (str[i] == '.'  || 
		 	 str[i] == '/'  || 
			 str[i] == ' '  || 
			 str[i] == '"'  || 
			 str[i] == ']'  || 
			 str[i] == '_'  ||
			 str[i] == '-'  ||
			 str[i] == '\'' ||
			 str[i] == '#'  ||
		    str[i] == '<'  ||
		    str[i] == '>'  ||
			 str[i] == '@'  ||
			 str[i] == '~'  ||
			 str[i] == '('  ||
			 str[i] == ')'  ||
		    str[i] == '$'  ||
			 str[i] == '`') continue;

		str[newplace++] = str[i];
	}

	str[newplace] = '\0';
}

//get the value of the a pgntag from a pgnheader <str>
//if str contains [White "hello"], output contains hello
//after calling this function
bool getPGNTag(char* str, char* tag, char *output)
{
   if (str[0] == '[' && beginsWith(str+1, tag))
	{
		str += strlen(tag) + 1;
		while (*str == ' ') str++;
		if (*str != '"') return false;
		str++;

		strcpy(output, str);

		rtrim(output, ']');
		rtrim(output, ' ');
		rtrim(output, '"');

		return true;
	}
	else
	{
		return false;
	}
}

//add a player to the player list
//we don't look for * characters in the name of the player
void addPlayerICC(char* str, nameListType* list)
{
	int lengte;

	rtrim(str);
	ltrim(str);

	//look for an [ in the string
    char* pos = strstr(str, " ");

	if (pos != NULL)
	{
		*pos = '\0';
		pos++;
		addPlayerICC(pos, list);
	}

	lengte = (int)strlen(str);
	
	//maximum name lengte reached?
	if (lengte >= MAX_NAME_LENGTH)
	{
		cerr << "Error: name too long" << endl;
		cerr << "name was: " << str << endl;
		exit(1);
	}

	//maximum in list reached?
	if (list->aantal >= MAX_LIST)
	{
		cerr << "Error: maximum in list reached" << endl;
		cerr << "maximum of the list is: " << MAX_LIST << endl;
		exit(1);
	}

	//copy the data
    nameRecordType *rec = &list->data[list->aantal];

	strcpy(rec->name, str);
	rec->start_star = false;
	rec->end_star = false;

	list->aantal++;
}

//add a playername <str> to a playerlist <list>
//if <str> contains * at the beginning or at the end 
//we threat this in a special way.
//
//if <str> contains for example Fritz[1-4] then 
//Fritz1, Fritz2, Fritz3 and Fritz4 is added
void addPlayer(char* str, nameListType* list)
{
	bool bStart = false;
	bool bEnd = false;
	int lengte = (int)strlen(str);

	//look for an [ in the string to see if we have a 
	//string in the form Fritz[1-8]*
   char* pos = strstr(str, "[");

	//if we find [ in the string ... 
	//then recursive make all options
	if (pos != NULL)
	{
		if (*(pos+2) == '-' && *(pos+4) == ']')
		{
			char fromchar = *(pos+1);
			char tochar   = *(pos+3);

			if (fromchar < tochar)
			{
				strcpy(pos+1, pos+5);
			}

			while (fromchar <= tochar)
			{
				*pos = fromchar;
				addPlayer(str, list);

				fromchar++;
			}

			return;
		}
		else
		{
			cerr << "Don't understand" << str << endl;
			exit(1);
		}
	}


	//start?
	if (*str == '*') 
	{
		bStart = true; 
		str++;
		lengte--;
	}

	//end?
	if (str[lengte-1] == '*') 
	{
		bEnd = true;
		lengte--;
	}

	//maximum name lengte reached?
	if (lengte >= MAX_NAME_LENGTH)
	{
		cerr << "Error: name too long" << endl;
		cerr << "name was: " << str << endl;
		exit(1);
	}

	//maximum in list reached?
	if (list->aantal >= MAX_LIST)
	{
		cerr << "Error: maximum in list reached" << endl;
		cerr << "maximum of the list is: " << MAX_LIST << endl;
		exit(1);
	}

	//copy the data
    nameRecordType *rec = &list->data[list->aantal];

	strncpy(rec->name, str, lengte);
	rec->name[lengte] = '\0';
	rec->start_star = bStart;
	rec->end_star = bEnd;

	list->aantal++;

	//delete trailing spaces
	rtrim(rec->name);
}

//read a list from a file <strfilename>
//the list is kept in <list>
void readList(char* strFileName, nameListType* list, bool have_more_on_one_line = false)
{
	cerr << "Reading " << strFileName << "..." << endl;

	list->aantal = 0;

	FILE *f = fopen(strFileName, "r");

	if (f == NULL)
	{
		cerr << "Error: During opening " << strFileName << endl;
		exit(1);
	}

	while (!feof(f))
	{
		if (fgets(line, MAXBUF_SIZE, f) == NULL) continue;

		if (line[0] == '#') continue;

		char* str = (char*)line;
        
		rtrim(str);

		if (have_more_on_one_line) 
		{
			addPlayerICC(str, list); 
		}
		else
		{
			addPlayer(str, list);
		}
	}
}

//print a playerlist
void printList(nameListType* list)
{
	for (int i=0; i<list->aantal; i++)
	{
		nameRecordType *rec = &list->data[i];

		if (rec->start_star) cout << "*";

		cout << rec->name;

		if (rec->end_star) cout << "*";

		cout << endl;
	}
}

//is a name <str> in the list <list> ?
bool isInList(char* str, nameListType* list)
{
	rtrim(str);

	for (int i=0; i<list->aantal; i++)
	{
		nameRecordType *rec = &list->data[i];

		if (rec->start_star && rec->end_star)
		{
			//we have to look something in the middle
			if (strstr(str, rec->name) != NULL) 
			{
				return true;
			}
		}
		else  if (rec->start_star)
		{
			//we have to look something at the end
			if (endsWith(str, rec->name))
			{
				return true;
			}
		}
		else if (rec->end_star)
		{
			//we have to look something in the begin
            if (beginsWith(str, rec->name))
			{
				return true;
			}
		}
		else if (strcmp(str, rec->name) == 0) 
		{
			return true;
		}
	}

	return false;
}

//for example Roman {2727}
void removeEloRatingFromPlayerName(char *playername)
{
	char *pos = strstr(playername, "{");
	char *start = pos;

	if (pos != NULL)
	{
		pos++;

		while (*pos >= '0' && *pos <= '9') pos++;

		if (*pos == '}')
		{
			pos++;
         strcpy(start, pos);
		}
	}

	rtrim(playername);
}

eEventType determineEventType()
{
	if (beginsWith(pgngame.event, "ICS")) return ICC_EVENT;
	if (beginsWith(pgngame.event, "ICC")) return ICC_EVENT;
	if (beginsWith(pgngame.event, "mmeics")) return ICC_EVENT;
	if (beginsWith(pgngame.site, "ICS")) return ICC_EVENT;
	if (beginsWith(pgngame.site, "ICC")) return ICC_EVENT;
	if (beginsWith(pgngame.site, "Internet Chess Club")) return ICC_EVENT;
	if (beginsWith(pgngame.site, "chessclub.com")) return ICC_EVENT;
	if (beginsWith(pgngame.site, "mmeics")) return ICC_EVENT;
	if (beginsWith(pgngame.event, "FICS")) return FICS_EVENT;
	if (beginsWith(pgngame.event, "fics")) return FICS_EVENT;
   if (beginsWith(pgngame.site, "fics")) return FICS_EVENT;
	if (beginsWith(pgngame.site, "FICS")) return FICS_EVENT;
	if (beginsWith(pgngame.site, "freechess.org")) return FICS_EVENT;
	if (strstr(pgngame.site, "rated")) return ICC_EVENT;
	if (strstr(pgngame.site, "Rated")) return ICC_EVENT;
	return REST_EVENT;
}

//determine the type of a player
void setPlayerData(playerDataType* p)
{
	//make an upper string
	strcpy(p->uppered, p->name);
	upper(p->uppered);

	if (beginsWith(p->uppered, "COMP: "))
	{
		p->type = COMP_PLAYER;
		return;
	}

	if (beginsWith(p->uppered, "COMP "))
	{
		p->type = COMP_PLAYER;
		return;
	}

	if (endsWith(p->uppered, "(C)"))
	{
		p->type = COMP_PLAYER;
		return;
	}

	if (endsWith(p->uppered, "<COMPUTER>"))
	{
		p->type = COMP_PLAYER;
		return;
	}

	//strip the upper string and make an
	//stripped and uppered string
	strcpy(p->stripped_and_uppered, p->uppered);
	stripCharsFromPlayerName(p->stripped_and_uppered);

	strcpy(p->cleaned_from_rating, p->name);
	removeEloRatingFromPlayerName(p->cleaned_from_rating);

	if ( strcmp(p->stripped_and_uppered, "BYE") == 0 ) 
	{
		p->type = COMP_BYE;
		return;
	}
	
   pgngame.eventType = determineEventType();

	if (pgngame.eventType == ICC_EVENT)
	{
		if ( isInList(p->cleaned_from_rating, &computers_icc) ) 
		{
			p->type = COMP_PLAYER;
			return;
		}
		if ( isInList(p->cleaned_from_rating, &humans_icc_titled) ) 
		{
			p->type = HUMAN_PLAYER;
			return;
		}
		if ( isInList(p->cleaned_from_rating, &humans_icc_rest) ) 
		{
			p->type = HUMAN_PLAYER;
			return;
		}
	}
	else if (pgngame.eventType == FICS_EVENT)
	{
		if ( isInList(p->cleaned_from_rating, &computers_fics) ) 
		{
			p->type = COMP_PLAYER;
			return;
		}
		if ( isInList(p->cleaned_from_rating, &humans_fics) ) 
		{
			p->type = HUMAN_PLAYER;
			return;
		}
	}
	else
	{
		if ( isInList(pgngame.site, &adv_events) ) 
		{
			p->type = ADVANCED_PLAYER;
			return;
		}
		if ( isInList(pgngame.event, &adv_events) ) 
		{
			p->type = ADVANCED_PLAYER;
			return;
		}
		if ( isInList(p->name, &advanced_chess) ) 
		{
			p->type = ADVANCED_PLAYER;
			return;
		}
		if ( isInList(p->stripped_and_uppered, &computers) ) 
		{
			p->type = COMP_PLAYER;
			return;
		}
		if ( isInList(p->stripped_and_uppered, &humans) ) 
		{
			p->type = HUMAN_PLAYER;
			return;
		}
		if ( isInList(p->stripped_and_uppered, &computers_when_opponent_is) ) 
		{
			p->type = COMP_MAYBE;
			return;
		}
	}

	p->type = UNKNOWN_PLAYER;
}

//print a ePlayerType
void printPlayerType(playerDataType* p)
{
	if (p->type == COMP_PLAYER) 
	{
		cout << "(C)";
	}
	else if (p->type == HUMAN_PLAYER) 
	{
		cout << "(H)";
	}
	else 
	{
		cout << "(?)";
	}
}

void testplayername(char *str)
{
	playerDataType p;
	strcpy(p.name, str);

	cout << str << '=';

	setPlayerData(&p);

	printPlayerType(&p);

	cout << endl;

	exit(1);
}

//print shortly the main headers of a pgngame
//we can use this after reading all headers of a pgn game
void printShortHeadersTags()
{
	//both computers?
	cout << '[' << pgngame.white.name << ']';

	printPlayerType(&pgngame.white);

	cout << "-";

	cout << '[' << pgngame.black.name << ']';

	printPlayerType(&pgngame.black);

	cout << ' ' << pgngame.event << endl;
}

//print all headers we found
void printAllHeadersTags()
{
	//both computers?
	for (int i=0; i<pgngame.tags.aantal; i++)
	{
		fprintf(active, "%s\n", pgngame.tags.data[i]);
	}
}

bool isCorrespondenceEvent(char *str)
{
	if (strstr(str, "corr ") != NULL) return true;
	if (strstr(str, "Corr ") != NULL) return true;
	if (strstr(str, "corr.") != NULL) return true;
	if (strstr(str, "Corr.") != NULL) return true;
	if (strstr(str, "cr ") != NULL) return true;
	if (strstr(str, "cr.") != NULL) return true;
	if (strstr(str, "korr.") != NULL) return true;
	if (strstr(str, "Korr.") != NULL) return true;

	return false;
}

void handleMaybe(ePlayerType& thistype, ePlayerType& opptype)
{
	if (isInList(pgngame.event, &comp_events))
	{
		thistype = COMP_PLAYER;
		return;
	}

	if (isInList(pgngame.event, &human_events))
	{
		thistype = HUMAN_PLAYER;
		return;
	}
	
	//we have a human as opponent ...
	//the game is played before 1950 .. so this can't a 
	//computer vs human game
	if (opptype == HUMAN_PLAYER && pgngame.year != 0 && pgngame.year < 1950)
	{
		thistype = HUMAN_PLAYER;
		return;
	}

	//we have a human as opponent ...
	//the game is played before 1970 .. so this can't a 
	//computer vs computer game
	if (opptype == COMP_PLAYER && pgngame.year != 0 && pgngame.year < 1970)
	{
		thistype = HUMAN_PLAYER;
		return;
	}

	if (isCorrespondenceEvent(pgngame.event))
	{
		thistype = HUMAN_PLAYER;
		return;
	}

	if (thistype == COMP_MAYBE)
	{
		if (opptype == COMP_PLAYER)
		{
			//we guess this is a computer vs computer game...
			thistype = COMP_PLAYER;
		}
		else if (opptype == COMP_MAYBE)
		{
			//we guess this is a comp_vs_comp game
			thistype = COMP_PLAYER;
			opptype = COMP_PLAYER;
		}
		else
		{
			thistype = UNKNOWN_PLAYER;
		}
	}
}

//determine the type of both players
void determineePlayerTypes()
{
	bool whiteComp = false;

	if (strcmp(pgngame.white.name, "M") == 0)
	{
		char *pWhite = (char*)&pgngame.white.name;
		char *pBlack = (char*)&pgngame.black.name;

		if (beginsWithAndDelete(pBlack, "Chess 6 - "))
		{
			strcpy(pgngame.white.name, "M Chess 6");
			strcpy(pgngame.black.name, pBlack);
		}
		else if (beginsWithAndDelete(pBlack, "Chess 4 - "))
		{
			strcpy(pgngame.white.name, "M Chess 4");
			strcpy(pgngame.black.name, pBlack);
		}
	}

	if (beginsWith(pgngame.white.name, "Hiarcs3 486/50 Junior"))
	{
		strcpy(pgngame.white.name, "Hiarcs 3");
		strcpy(pgngame.black.name, "Junior");
	}

	if (beginsWith(pgngame.white.name, "Junior 486/50 Hiarcs3"))
	{
		strcpy(pgngame.white.name, "Junior");
		strcpy(pgngame.black.name, "Hiarcs 3");
	}

	//what kind of player is white?
	setPlayerData(&pgngame.white);
	setPlayerData(&pgngame.black);

	//
	//look if one of the players had a bye
	//
	if (pgngame.white.type == COMP_BYE)
	{
		pgngame.white.type = pgngame.black.type;
	}

	if (pgngame.black.type == COMP_BYE)
	{
		pgngame.black.type = pgngame.white.type;
	}

	//
	//if white was maybe a computer player then make it or not
	//
	if (pgngame.white.type == COMP_MAYBE)
	{
		handleMaybe(pgngame.white.type, pgngame.black.type);
	}

	if (pgngame.black.type == COMP_MAYBE)
	{
		handleMaybe(pgngame.black.type, pgngame.white.type);
	}
}

//determine the type of the game
void determineeGameType()
{
	if (pgngame.white.type == ADVANCED_PLAYER ||
		pgngame.black.type == ADVANCED_PLAYER)
	{
		active = file_advanced_chess;
		pgngame.eGameType = ADVANCED_GAME;
		return;
	}

	if (pgngame.white.type == COMP_PLAYER && 
		pgngame.black.type == COMP_PLAYER)
	{
		active = file_comp_vs_comp;
		pgngame.eGameType = COMP_VS_COMP;
		return;
	}

	if (pgngame.white.type == COMP_PLAYER && 
		pgngame.black.type == HUMAN_PLAYER)
	{
		active = file_comp_vs_human;
		pgngame.eGameType = COMP_VS_HUMAN;
		return;
	}

	if (pgngame.white.type == HUMAN_PLAYER && 
		pgngame.black.type == COMP_PLAYER)
	{
		active = file_comp_vs_human;
		pgngame.eGameType = COMP_VS_HUMAN;
		return;
	}

	if (pgngame.white.type == COMP_PLAYER && 
		pgngame.black.type == UNKNOWN_PLAYER)
	{
		active = file_comp_vs_rest;
		pgngame.eGameType = COMP_VS_UNKNOWN;
		return;
	}

	if (pgngame.white.type == UNKNOWN_PLAYER && 
		pgngame.black.type == COMP_PLAYER)
	{
		active = file_comp_vs_rest;
		pgngame.eGameType = COMP_VS_UNKNOWN;
		return;
	}

	active = file_rest_vs_rest;
	pgngame.eGameType = REST_VS_REST;
}

//reset all fields of a pgn game
void resetPgnGame()
{
	memset(&pgngame, 0, sizeof(pgngame) );

	//sometimes pgn's doesn't start with games
	//then put everything to file_rest_vs_rest
	active = file_rest_vs_rest;
}


//look how many times character 'c' occures in the string 'str'
int getcountchar(char *str, char c)
{
	int number = 0;

	for (int i=0; i<(int)strlen(str); i++)
	{
		if (str[i] == c) number++;
	}

	return number;
}

//get a string part of a string seperated with 'c'
//the first part is 0, the 2nd part 1, etc
void gettoken(char *str, char c, int searchnumber, char *buffer)
{
	int number = 0;
	char *strbuffer = buffer;

	for (int i=0; i<(int)strlen(str); i++)
	{
		if (str[i] == c) 
		{
			number++; 
		}
		else if (number == searchnumber)
		{
			*strbuffer = str[i];
			strbuffer++;
		}
	}

	*strbuffer = '\0';
}

//swap two integervalues <a> and <b>
void swap(int a, int b)
{
	int tmp;
		
    tmp = a;
	a = b;
	b = tmp;
}

//take some characters away from a tagfield
void rtrimtag(char *s)
{
   for (int lengte = (int)strlen(s)-1; lengte >= 0; lengte--)
   {
      if (s[lengte] == '\n' || 
		  s[lengte] == '\t' || 
		  s[lengte] == 10   ||
		  s[lengte] == 13   ||
		  s[lengte] == ' '  ||
		  s[lengte] == '"'  ||
		  s[lengte] == ']')
      {
         s[lengte] = '\0';
      }
      else
      {
         break;
      }
   }
}

//does the string <str> contain only numbers of ???
bool isNumbersOrQuestionMarkOnly(char *str)
{
	int lengte = (int)strlen(str);

	for (int i=0; i<lengte; i++)
	{
		if (str[i] == '?') continue;
		if (str[i] < '0' || str[i] > '9') return false;
	}

	return true;
}

void replacechar(char *str, char from, char to)
{
    for (int i=0; i<(int)strlen(str); i++)
	{
		if (str[i] == from)
		{
			str[i] = to; 
		}
	}
}



//does the string <str> contain a legal date? 
bool isLegalDate(char *str)
{
	int number;
	char temp[100];

	pgngame.year=0;
	pgngame.month=0;
	pgngame.day=0; 

	//look for '"'
	char *pos = strstr(str, "\"");

	//we didn't found a '"'
	if (pos == NULL) return false;

	//skip '"'
	pos++;

	//do we have a closing '"' immediatly? 
	//then stop
	if (*pos == '"') return true;

	if (beginsWith(pos, "28 oktober 2001")) strcpy(pos, "2001.10.28");
	if (beginsWith(pos, "28 0ktober 2001")) strcpy(pos, "2001.10.28");
	if (beginsWith(pos, "27 oktober 2001")) strcpy(pos, "2001.10.27");
	if (beginsWith(pos, "27 oktober 2002")) strcpy(pos, "2002.10.27");
	if (beginsWith(pos, "5-6 OCT 2002")) strcpy(pos, "2002.10.05");
	if (beginsWith(pos, "5-6 oct 2002")) strcpy(pos, "2002.10.05");

	rtrimtag(pos);

	//replace all minus with points
	replacechar(str, '-', '.');

	//how much '.' do we have?
	number = getcountchar(str, '.');

	//we found two so the format must be yyyy.mm.dd
	if (number == 2)
	{
		gettoken(pos, '.', 0, temp);
		if (!isNumbersOrQuestionMarkOnly(temp)) return false;
		pgngame.year = atoi(temp);

		gettoken(pos, '.', 1, temp);
		if (!isNumbersOrQuestionMarkOnly(temp)) return false;
		pgngame.month = atoi(temp);

		gettoken(pos, '.', 2, temp);
		if (!isNumbersOrQuestionMarkOnly(temp)) return false;
		pgngame.day = atoi(temp);
	}
	else if (number == 0)
	{
		//the format must be '????"
		if (!isNumbersOrQuestionMarkOnly(pos)) return false;
		pgngame.year = atoi(str);
	}
	else
	{
		//we have an illegal date
		return false;
	}

	if (pgngame.day > 1000)	
	{
		swap(pgngame.day, pgngame.year);
	}

	//year
	if (pgngame.year != 0)
	{
		pos += sprintf(pos, "%04d", pgngame.year);
	}
	else
	{
		pos += sprintf(pos, "%s", "????");
	}

	//month
	if (pgngame.month != 0)
	{
		pos += sprintf(pos, ".%02d", pgngame.month);
	}
	else
	{
		pos += sprintf(pos, ".%s", "??");
	}

	//day
	if (pgngame.day != 0)
	{
		pos += sprintf(pos, ".%02d]\"", pgngame.day);
	}
	else
	{
		pos += sprintf(pos, ".%s]\"", "??");
	}

	return true;
}

//look for the last occurence of character <c> in string <str>
char *strlastchar(char *str, char c)
{
	int lengte = (int)strlen(str);

	for (int i = lengte-1; i >= 0; i--)
	{
		if (str[i] == c) return str+i;
	}

	return NULL;
}

//some ICC games prior to 1998 have in the header
//the date ... like
//[Event "ICC 0 2 07/03/97"] or
//[Event "ICC 5 3 09/06/1998"]
//[Event "ICC u 0 2 06/04/1998
void geticcdateFromEvent(char *str)
{
	char *eventdate;
	int year=0, month=0, day=0; 
	char temp[100];

	if (!beginsWith(pgngame.event, "ICC")) return;

	//take the date from the event
	eventdate = strlastchar(pgngame.event, ' ');
	if (eventdate == NULL) return;

	//skip the space
	eventdate++;

	//if we don't have two times a '/' then stop
	if (getcountchar(eventdate, '/') != 2) return;

	//get the month
	gettoken(eventdate, '/', 0, temp);
	if (!isNumbersOrQuestionMarkOnly(temp)) return;
	month = atoi(temp);

	//get the day
	gettoken(eventdate, '/', 1, temp);
	if (!isNumbersOrQuestionMarkOnly(temp)) return;
	day = atoi(temp);

	//get the year
	gettoken(eventdate, '/', 2, temp);
	if (!isNumbersOrQuestionMarkOnly(temp)) return;
	year = atoi(temp);

	//if the year is smaller as 100 then add 1900
	if (year < 100) year += 1900;

	//print the date
	rtrim(str, ' ');
	rtrim(str, ']');
	rtrim(str, '"');
	eventdate = strlastchar(str, ' ');
	sprintf(eventdate, "%s", "\"]");

	//get the date
	sprintf(pgngame.date_icc_fixed, "%04d.%02d.%02d", year, month, day);
}

//check an icc date and copy the date from the event when
//necesarry
void checkDate(char *str)
{
	if (!isLegalDate(str))
	{
		cerr << "Warning: skipped illegal datetag of " << str << endl;
		strcpy(str, "[Date \"????.??.??\"]");
	}

	//we don't have a date
	if (strstr(str, "????.??.??") != NULL)
	{
		//we have a iccdate
		if (pgngame.date_icc_fixed[0] != '\0')
		{
         sprintf(str, "[Date \"%s\"]", pgngame.date_icc_fixed);
		}
	}
}

void processHeaderTag(char *str)
{
	char output[100];

	//delete \" from the string
	deleteAllFromString(str, "\\\"");

	if (getPGNTag(str, "White", output)) 
	{
		strcpy(pgngame.white.name, output);
		//removeEloRatingFromPlayerName(pgngame.white.name);
	}
	else if (getPGNTag(str, "Black", output)) 
	{
		strcpy(pgngame.black.name, output);
		//removeEloRatingFromPlayerName(pgngame.black.name);
	}
	else if (getPGNTag(str, "Site", output)) 
	{
		strcpy(pgngame.site, output);
	}
	else if (getPGNTag(str, "Event", output)) 
	{
		strcpy(pgngame.event, output);
		geticcdateFromEvent(str);
	}


	//skip illegal dates
   if (beginsWith(str, "[Date "))
	{
		checkDate(str);
	}

	strcpy(pgngame.tags.data[pgngame.tags.aantal], str);
	pgngame.tags.aantal++;
}

//look if this is a tag
//we can't just look at [ since some games have this form in the evaluation, [%eval
bool isPGNTag(char *str)
{
	//skip first '[' 
	if (*str != '[') return false;
	str++; 

	//skip all characters
	while ((*str >= 'a' && *str <= 'z') || (*str >= 'A' && *str <= 'Z'))
	{
		str++;
	}

	//skip space
	while (*str == ' ') str++;

	if (*str != '"') return false;
	str++;

	return true;
}


void processLine(char *str)
{
	static bool bInHeader = false;
	static bool bPrintThisGame = false;

	rtrim(str);

	//skip empty string
	if (bInHeader && str[0] == '\0') return;

   //is header tag
	if (isPGNTag(str))
	{
		//we find new headers ... reset all old ones
		if (!bInHeader)
		{
			resetPgnGame();
			bInHeader = true;
		}

		//process this header
		processHeaderTag(str);
	}
	else
	{
		if (bInHeader)
		{
			determineePlayerTypes();
			determineeGameType();

			//since this is the first field what is not 
			//a header ... and we think we are still in the
			//header tags ... print all headers
			printAllHeadersTags();

			bInHeader = false;
		}

		//print the last read string (moves)
		fprintf(active, "%s\n", str);
	}
}

FILE *open_or_create(char *filename)
{
	FILE *f = fopen(filename, "a+");
	if (f == NULL)
	{
		cerr << "Error: During creating or opening file '" << filename << "'" << endl;
		exit(1);
	}

	return f;
}

void info()
{
	cerr << "Usage: pgncompfind <pgnfile>" << endl;
	cerr << endl;
	cerr << "Comments to Michel Langeveld (michel.langeveld@xs4all.nl)" << endl;
	cerr << "Version 1.01, okt 2003" << endl;
	cerr << endl;
	cerr << "Parses a pgnfile and output the following" << endl;
	cerr << "comp-vs-comp.pgn" << endl;
	cerr << "comp-vs-human.pgn" << endl;
	cerr << "comp-vs-rest.pgn" << endl;
	cerr << "rest-vs-rest.pgn" << endl;
}

void readLists()
{
	//
	//read the lists
	//
	readList("humans.cfg", &humans, false);
	readList("humans_icc_rest.cfg", &humans_icc_rest, false);
	readList("human_events.cfg", &human_events, false);

	readList("computers.cfg", &computers, false);
	readList("computers_when_opponent_is.cfg", &computers_when_opponent_is, false);
	readList("computers_icc.cfg", &computers_icc, false);
	readList("comp_events.cfg", &comp_events, false);
	readList("advanced_chess.cfg", &advanced_chess, false);
	readList("adv_events.cfg", &adv_events, false);

	//
	//read some lists that can have more as one playername 
	//on a line
	//
	readList("computers_fics.cfg", &computers_fics, true);
	readList("humans_icc_titled.cfg", &humans_icc_titled, true);
	readList("humans_fics.cfg", &humans_fics, true);
}

int main(int argc, char* argv[])
{
	if (argc <= 1)
	{
		info();
		exit(1);
	}

	readLists();

	//strcpy(pgngame.event, "ICS blabla");
	//testplayername("Roman {1232}");

	//we open it "rb" since some files contain a CTRL+Z
	//that will give problems. 
	FILE *f = fopen(argv[1], "rb");
	if (f == NULL)
	{
		cerr << "Error: During opening " << argv[1] << endl;
		exit(1);
	}

	file_comp_vs_comp   = open_or_create("comp_vs_comp.pgn");
	file_comp_vs_human  = open_or_create("comp_vs_human.pgn");
	file_comp_vs_rest   = open_or_create("comp_vs_rest.pgn");
	file_rest_vs_rest   = open_or_create("rest_vs_rest.pgn");
	file_advanced_chess = open_or_create("advanced.pgn");
	
	resetPgnGame();

	for (;;)
	{
		if (fgets(line, MAXBUF_SIZE, f) == NULL) 
		{
			break;
		}

		char* str = (char*)line;

		processLine(str);
	}

	//
	// close all files
	//
	fclose(file_comp_vs_comp);
	fclose(file_comp_vs_human); 
	fclose(file_comp_vs_rest); 
	fclose(file_rest_vs_rest); 
	fclose(file_advanced_chess); 

	return 0;
}
